
% Microstructure model of Bandi et. al (2017), Ecta 

clear all;
close all;
clc;

addpath(genpath(pwd));

%% General settings

n = 2;                                    % N. of markets in which the efficient price is observed
q = 0.5;                                  % Variance of the efficient price
c = [0.2;0.2];                            % Transaction costs
pait = 0.7;                                 % Probability of arrival of informed traders 

Psi = [0.9, 0; 0, 0.2];

a1aux = [0.1;0.1];
a2aux = [0.1;0.1];

nTimeStamps = 4000;                                             % N. of timestamps
nSim = 1000;

pMiss = rand(n,nTimeStamps)<repmat([0;0],1,nTimeStamps);                % Prob. of missing values (consider only if pait = 0)

%% Build parameters

eyen = eye(n);
iota = diag(eyen);
zerosn = zeros(n);

T = [2*eyen-Psi, -eyen+Psi; eyen, zerosn];
R = [Psi, zerosn; zerosn, zerosn];
Q = [iota*iota'*q, zerosn;
    zerosn, zerosn];
M = [eyen,zerosn];

parTrue = [reshape(Psi,n^2,1);q;c];

%% Processing

parInit = parTrue*(1-0);
nPar = length(parInit);

beta = [ones(n-1,1),-eye(n-1)]';

for i = 1:nSim
    
    fprintf('Sim. n. %s of %s\n',num2str(i),num2str(nSim));
    
    % Simulation
    
    p = nan(1,nTimeStamps);
    z = nan(2*n,nTimeStamps);
    y = nan(n,nTimeStamps);
    
    U = rand(n,nTimeStamps);
    omega = sqrt(q)*randn(1,nTimeStamps-1);
    eta = [repmat(omega,n,1); zeros(n,nTimeStamps-1)];
    epsilon = double(rand(n,nTimeStamps)>0.5);
    epsilon(epsilon==0)=-1;
    
    p(1) = a1aux(1,1);
    a1 = [a2aux;a1aux];
    P1 = 0.0*eye(2*n);
    z(:,1) = a1 + sqrt(P1)*randn(2*n,1);
    for  j = 1:n
        if U(j,1)<=pait
            if abs(p(1)-M(j,:)*z(:,1))>c(j)
                y(j,1) = M(j,:)*z(:,1) + c(j)*(p(1)-M(j,:)*z(:,1)>c(j))-c(j)*(p(1)-M(j,:)*z(:,1)<-c(j));
            else
                y(j,1)=nan;
            end
        else
            y(j,1) = M(j,:)*z(:,1) + epsilon(j,1)*c(j);
        end
    end
    
    for t = 2:nTimeStamps
        p(t) = p(t-1) + eta(1,t-1);
        z(:,t) = T*z(:,t-1) + R*eta(:,t-1);
        for  j = 1:n
            if U(j,t)<=pait
                if abs(p(t)-M(j,:)*z(:,t))>c(j)
                    y(j,t) = M(j,:)*z(:,t) + c(j)*(p(t)-M(j,:)*z(:,t)>c(j))-c(j)*(p(t)-M(j,:)*z(:,t)<-c(j));
                else
                    y(j,t) = nan;
                end
            else
                y(j,t) = M(j,:)*z(:,t) + epsilon(j,t)*c(j);
            end
        end
    end
    
    x = z(1:n,:);
    
    if pait == 0
        y(pMiss) = nan;
    end
    
    % MLA
    P1In = 1*ones(2*n);
    a1In = zeros(2*n,1);
    parEst = estDegMLA(y,parInit,n,a1In,P1In);
    
    % IS (Hasbrouck 1995)
    
    p2 = 3;    a1 = [a2aux;a1aux];
    yInIS = nan(n,nTimeStamps);
    for j=1:n
        yInIS(j,:) = fillNans(y(j,:));
    end
        
    [isBNsupp, PiEst, GammaEst1, GammaEst2, Qest, psiEst] = computeIS2(yInIS,beta,p2);
    isBNrev = computeIS2(yInIS([2,1],:),beta,p2);
    isBound(1,1) = isBNsupp(1);
    isBound(2,1) = isBNrev(2);
    isBound(1,2) = isBNsupp(2);
    isBound(2,2) = isBNrev(1);
    
    
end

    
    
    
