function [x_filt,x_smooth,Q,R,c_conv,l]=EM(y,Q_init,R_init,maxiter,eps,trueQ,trueR,showplot)    

% EM algorithm with missings, only for diagonal R
% INPUTS:  y        = dxT data matrix with NaN of d subjects for T observations
%          Q_init   = initial value for innovation error variance
%          R_init   = initial value for observation error variance
%          maxiter  = max number of iterations allowed before convergence
%          eps      = min tolerance relative change in log-likelihood for convergence
%          trueQ    = true value of model innovation variance 
%          trueR    = true value of observation error variance
%          showplot = plots are (not) shown if 1 (0)
% OUTPUTS: x_filt   = filtered data at last iteration
%          x_smooth = smoothed data at last iteration
%          Q        = estimated model innovation variance at last iteration
%          R        = estimated observation error variance at last iteration
%          c_conv   = equal to 1 if convergence has been reached
%          l        = log-likelihood path
%          qVec     = vectorized innovation variance path
%          rVec     = vectorized observation error variance path

% preliminaries
[d,T]=size(y);
a = isnan(y)==0;        % index of observed values
c_conv=0;
Id = eye(d);
burnIn = d+1;

% memory allocation
l = NaN*ones(1,maxiter);
currfrobQ = NaN*ones(1,maxiter);
currfrobR = NaN*ones(1,maxiter);
deltalog = NaN*ones(1,maxiter);

% initialization
l_old = -10e10; 
Q = Q_init; R = R_init; 

if showplot
    figure;
end

for i=1:maxiter
    
    disp(i)
    tic
    
    % Kalman filtering and smoothing
    y(a==0)=NaN;
    [x_filt,x_smooth,V_smooth,Vt_smooth,loglik]=KF(y,Q,R);
    y(a==0)=0;
    
    % compute incomplete data log-likelihood
    l(i)=loglik;
   
    if showplot
        currfrobQ(i)=norm(Q-trueQ,'fro')/norm(trueQ,'fro');
        currfrobR(i)=norm(R-trueR,'fro')/norm(trueR,'fro');
        subplot(1,3,1),plot(currfrobQ(1:i)),title('||Q-Q_{true}||');
        subplot(1,3,2),plot(currfrobR(1:i)),title('||R-R_{true}||');
        subplot(1,3,3),plot(l(1:i)),title('Log-likelihood');
        drawnow
    end
    
    % E step
    
    S=zeros(d); 
    S10 = zeros(d); eps_smooth = zeros(d);
    
    for t=burnIn:T
        
        Dobs = [Id(a(:,t)==1,:); zeros(nnz(~a(:,t)),d)];
        Dback = [Id(a(:,t)==1,:); Id(a(:,t)==0,:)]';
        Dmiss = [zeros(nnz(a(:,t)),d); Id(a(:,t)==0,:)];
        
        auxY = Dobs*y(:,t);
        auxC = Dobs;
        
        S=S+x_smooth(:,t)*x_smooth(:,t)'+V_smooth(:,:,t);
        S10=S10+x_smooth(:,t)*x_smooth(:,t-1)'+Vt_smooth(:,:,t);
        
        eps_smooth=eps_smooth+Dback*((auxY-auxC*x_smooth(:,t))*(auxY-auxC*x_smooth(:,t))' + auxC*V_smooth(:,:,t)*auxC' + Dmiss*R*Dmiss' )*Dback';
    
    end
    
    S00 = S - x_smooth(:,T)*x_smooth(:,T)'-V_smooth(:,:,T) + x_smooth(:,burnIn-1)*x_smooth(:,burnIn-1)'+V_smooth(:,:,burnIn-1);
    S11 = S; 
    
    % M step
    
    BB = S10;
    AA = S00;
    CC = S11;
    
    R = diag(diag(eps_smooth/(T-burnIn+1)));
    Q = (CC-BB-BB'+AA)/(T-burnIn+1);
    
    % break for convergence
    deltalog(i)=abs(l(i)-l_old)/abs(l_old);
    if deltalog(i)<eps,
        str=int2str(i);
        disp(['break for convergence at iteration ', str])
        c_conv=1;
        break
    end
    l_old=l(i);
    toc
end

% display if missed convergence
if i==maxiter, disp('convergence not reached'); end

% reshaping for convergence before maxiter
if i < maxiter
    l=l(isnan(l)==0);
end