function [x_filt,x_smooth,V_smooth,Vt_smooth,logLik]=KF(y,Q,R)

% Kalman filter and smoother for the dynamic linear model
% y_t = C * x_t     + eta_t, eta_t~N(0,R)
% x_t = A * x_{t-1} + eps_t, eps_t~N(0,Q)

% INPUTS:  y         = dxT matrix of T observations for d subjects
%          Q         = dxd observational error matrix
%          R         = dxd innovation error matrix
%          A         = transition matrix
%          C         = measurement matrix
% OUTPUTS: x_pred    = predicted value of the latent process
%          x_smooth  = smoothed value of the latent process
%          V_pred    = variance of the prediction error
%          V_smooth  = variance of the smoothing error
%          Vt_smooth = one-lag autocovariance of the smoothing error
%          D         = instrumental matrix expressing the presence of missings
%          V_filt    = variance of the filtering error

%warning('off','all')

% preliminaries and memory allocation
[d,T]=size(y);
a = isnan(y)==0;        % index of observed values
Id = eye(d);
x_pred = zeros(d,T); x_filt = zeros(d,T); x_smooth = zeros(d,T);
V_filt = zeros(d,d,T); V_pred = zeros(d,d,T); V_smooth = zeros(d,d,T);
Vt_smooth = zeros(d,d,T); J = zeros(d,d,T);
loglik = zeros(1,T);

y(a==0)=0;

% Diffuse initialization (see Durbin & Koopman, 2001)
x_pred(:,1) = zeros(d,1);
V_pred(:,:,1) = 10^3*eye(d);

D = [Id(a(:,1)==1,:); zeros(nnz(~a(:,1)),d)];

auxY = D*y(:,1);
auxC = D;
auxR = D*R*D';
auxR(find(diag(~diag(auxR)))) = 1;

F = auxC*V_pred(:,:,1)*auxC' + auxR;
K = V_pred(:,:,1)*auxC'/F;

v = auxY-auxC*x_pred(:,1);
x_filt(:,1) = x_pred(:,1) + K*v;
V_filt(:,:,1) = V_pred(:,:,1) - K*auxC*V_pred(:,:,1);

loglik(1) = -0.5*(log(abs(det(F)))+v'/F*v);

for t = 2:T
    
    D = [Id(a(:,t)==1,:); zeros(nnz(~a(:,t)),d)];
    
    auxY = D*y(:,t);
    auxC = D;
    auxR = D*R*D';
    auxR(find(diag(~diag(auxR)))) = 1;
    
    x_pred(:,t) = x_filt(:,t-1);
    V_pred(:,:,t) = V_filt(:,:,t-1) + Q;
    
    F = auxC*V_pred(:,:,t)*auxC' + auxR;
    K = V_pred(:,:,t)*auxC'/F;
    
    v = auxY-auxC*x_pred(:,t);
    x_filt(:,t) = x_pred(:,t) + K*v;
    V_filt(:,:,t) = V_pred(:,:,t) - K*auxC*V_pred(:,:,t);
    
    loglik(t) = -0.5*(log(abs(det(F)))+v'/F*v);

end

% smoothing initialization
x_smooth(:,T)=x_filt(:,T);
V_smooth(:,:,T)=V_filt(:,:,T);
Vt_smooth(:,:,T)=(Id-K*auxC)*V_filt(:,:,T-1);
J(:,:,T-1)=V_filt(:,:,T-1)/V_pred(:,:,T);
x_smooth(:,T-1)=x_filt(:,T-1)+J(:,:,T-1)*(x_smooth(:,T)-x_filt(:,T-1));
V_smooth(:,:,T-1)=V_filt(:,:,T-1)+J(:,:,T-1)*(V_smooth(:,:,T)-V_pred(:,:,T))*J(:,:,T-1)';

% Rauch recursions
for t=T-1:-1:2
    J(:,:,t-1)=V_filt(:,:,t-1)/V_pred(:,:,t);
    x_smooth(:,t-1)=x_filt(:,t-1)+J(:,:,t-1)*(x_smooth(:,t)-x_filt(:,t-1));
    V_smooth(:,:,t-1)=V_filt(:,:,t-1)+J(:,:,t-1)*(V_smooth(:,:,t)-V_pred(:,:,t))*J(:,:,t-1)';
    Vt_smooth(:,:,t)=V_filt(:,:,t)*J(:,:,t-1)'+J(:,:,t)*(Vt_smooth(:,:,t+1)-V_filt(:,:,t))*J(:,:,t-1)';
end

logLik = sum(loglik(d+1:T));
