function [x_filt,x_smooth,V_smooth,Vt_smooth,logLik]=Ksmoother_miss_fastNew(y,Q,R,A,C)

% Kalman filter and smoother for the dynamic linear model
% y_t = C * x_t     + eta_t, eta_t~N(0,R)
% x_t = A * x_{t-1} + eps_t, eps_t~N(0,Q)

% INPUTS:  y         = dxT matrix of T observations for d subjects
%          Q         = dxd observational error matrix
%          R         = dxd innovation error matrix
%          A         = transition matrix
%          C         = measurement matrix
% OUTPUTS: x_pred    = predicted value of the latent process
%          x_smooth  = smoothed value of the latent process
%          V_pred    = variance of the prediction error
%          V_smooth  = variance of the smoothing error
%          Vt_smooth = one-lag autocovariance of the smoothing error
%          D         = instrumental matrix expressing the presence of missings
%          V_filt    = variance of the filtering error

%warning('off','all')

% preliminaries and memory allocation
[d,T]=size(y);
a = isnan(y)==0;        % index of observed values
dx = 2*d;
Idx = eye(dx);
Id = eye(d);
x_filt = zeros(dx,T); x_smooth = zeros(dx,T);
V_filt = zeros(dx,dx,T); V_pred = zeros(dx,dx,T); V_smooth = zeros(dx,dx,T);
Vt_smooth = zeros(dx,dx,T); 
loglik = zeros(T,1);

y(a==0)=0;

% Diffuse initialization (see Durbin & Koopman, 2001)
%x_pred(:,1) = zeros(dx,1);
V_pred(:,:,1) = 10^12*eye(dx);
x_pred = repmat(y(:,1),2,1);

D = [Id(a(:,1)==1,:); zeros(nnz(~a(:,1)),d)];

auxY = D*y(:,1);
auxC = D*C;
auxR = D*R*D';
auxR(find(diag(~diag(auxR)))) = 1;

F = auxC*V_pred(:,:,1)*auxC' + auxR;
K = V_pred(:,:,1)*auxC'/F;

v = auxY-auxC*x_pred;
x_filt(:,1) = x_pred + K*v;
V_filt(:,:,1) = V_pred(:,:,1) - K*auxC*V_pred(:,:,1);

loglik(1) = -0.5*(log(abs(det(F)))+v'/F*v);

for t = 2:T
    
    D = [Id(a(:,t)==1,:); zeros(nnz(~a(:,t)),d)];
    
    auxY = D*y(:,t);
    auxC = D*C;
    auxR = D*R*D';
    auxR(find(diag(~diag(auxR)))) = 1;
    
    x_pred = A*x_filt(:,t-1);
    V_pred(:,:,t) = A*V_filt(:,:,t-1)*A' + Q;
    
    F = auxC*V_pred(:,:,t)*auxC' + auxR;
    K = V_pred(:,:,t)*auxC'/F;
    
    v = auxY-auxC*x_pred;
    x_filt(:,t) = x_pred + K*v;
    V_filt(:,:,t) = V_pred(:,:,t) - K*auxC*V_pred(:,:,t);
    
    loglik(t) = -0.5*(log(abs(det(F)))+v'/F*v);

end

% smoothing initialization
x_smooth(:,T)=x_filt(:,T);
V_smooth(:,:,T)=V_filt(:,:,T);
Vt_smooth(:,:,T)=(Idx-K*auxC)*A*V_filt(:,:,T-1);
J=V_filt(:,:,T-1)*A'/V_pred(:,:,T);
x_smooth(:,T-1)=x_filt(:,T-1)+J*(x_smooth(:,T)-A*x_filt(:,T-1));
V_smooth(:,:,T-1)=V_filt(:,:,T-1)+J*(V_smooth(:,:,T)-V_pred(:,:,T))*J';

% Rauch recursions
for t=T-1:-1:2
    auxJ = J;
    J=V_filt(:,:,t-1)*A'/V_pred(:,:,t);
    x_smooth(:,t-1)=x_filt(:,t-1)+J*(x_smooth(:,t)-A*x_filt(:,t-1));
    V_smooth(:,:,t-1)=V_filt(:,:,t-1)+J*(V_smooth(:,:,t)-V_pred(:,:,t))*J';
    Vt_smooth(:,:,t)=V_filt(:,:,t)*J'+auxJ*(Vt_smooth(:,:,t+1)-A*V_filt(:,:,t))*J';
end

logLik = sum(loglik(d+1:end));