function [x_filt,x_smooth,Q,R,F,c_conv,normQ,normR,normF]=LLEM(y,Q_init,R_init,F_init,C,maxiter,eps,trueQ,trueR,trueF,showplot)    

% EM algorithm with missings, only for diagonal R
% INPUTS:  y        = dxT data matrix with NaN of d subjects for T observations
%          Q_init   = initial value for innovation error variance
%          R_init   = initial value for observation error variance
%          F_init   = initial value for transition matrix
%          C        = Observation matrix
%          maxiter  = max number of iterations allowed before convergence
%          eps      = min tolerance relative change in log-likelihood for convergence
%          trueQ    = true value of model innovation variance 
%          trueR    = true value of observation error variance
%          trueF    = true value of transition matrix
%          showplot = plots are (not) shown if 1 (0)
% OUTPUTS: x_filt   = filtered data at last iteration
%          x_smooth = smoothed data at last iteration
%          Q        = estimated model innovation variance at last iteration
%          R        = estimated observation error variance at last iteration
%          F        = estimated transition matrix at last iteration
%          c_conv   = equal to 1 if convergence has been reached
%          l        = log-likelihood path
%          qVec     = vectorized innovation variance path
%          rVec     = vectorized observation error variance path
%          lVec     = vectorized transition matrix path

% preliminaries
[d,T]=size(y);
a = isnan(y)==0;        % index of observed values
c_conv=0;
Id = eye(d);
P = [Id, zeros(d)];
burnIn = d+1;

%zeroIndex = find(trueF(1:d,1:d)-eye(d) == 0);

% memory allocation
l = NaN*ones(1,maxiter);
currfrobQ = NaN*ones(1,maxiter);
currfrobR = NaN*ones(1,maxiter);
currfrobF = NaN*ones(1,maxiter);
deltalog = NaN*ones(1,maxiter);

% initialization
l_old = -10e10; 
Q = Q_init; R = R_init; F = F_init; 

if showplot
    figure;
end

for i=1:maxiter
    
    % disp(i)
    % tic
    
    % Kalman filtering and smoothing
    y(a==0)=NaN;
    

    [x_filt,x_smooth,V_smooth,Vt_smooth,loglik]=Ksmoother_miss_fastSim(y,Q,R,F,C);

    y(a==0)=0;
    
    % compute incomplete data log-likelihood
    l(i)=loglik;
    
    % relevant plots
    currfrobQ(i)=norm(Q-trueQ,'fro')/norm(trueQ,'fro');
    currfrobR(i)=norm(R-trueR,'fro')/norm(trueR,'fro');
    currfrobF(i)=norm(F(1:d,d+1:2*d)-trueF(1:d,d+1:2*d),'fro')/norm(trueF(1:d,d+1:2*d),'fro');
    
    if showplot
        subplot(2,2,1),plot(currfrobQ(1:i)),title('||Q-Q_{true}||');
        subplot(2,2,2),plot(currfrobR(1:i)),title('||R-R_{true}||');
        subplot(2,2,3),plot(currfrobF(1:i)),title('||F-F_{true}||');
        subplot(2,2,4),plot(l(1:i)),title('Log-likelihood');
        drawnow
    end
    
    % E step
    
    S=zeros(2*d); 
    S10 = zeros(2*d); eps_smooth = zeros(d);
    
    for t=burnIn:T
        
        Dobs = [Id(a(:,t)==1,:); zeros(nnz(~a(:,t)),d)];
        Dback = [Id(a(:,t)==1,:); Id(a(:,t)==0,:)]';
        Dmiss = [zeros(nnz(a(:,t)),d); Id(a(:,t)==0,:)];
        
        auxY = Dobs*y(:,t);
        auxC = Dobs*C;
        
        S=S+x_smooth(:,t)*x_smooth(:,t)'+V_smooth(:,:,t);
%       S00 = S00 + x_smooth(:,t-1)*x_smooth(:,t-1)'+V_smooth(:,:,t-1);
%       S11 = S11 + x_smooth(:,t)*x_smooth(:,t)'+V_smooth(:,:,t);
        S10=S10+x_smooth(:,t)*x_smooth(:,t-1)'+Vt_smooth(:,:,t);
        
        eps_smooth=eps_smooth+Dback*((auxY-auxC*x_smooth(:,t))*(auxY-auxC*x_smooth(:,t))' + auxC*V_smooth(:,:,t)*auxC' + Dmiss*R*Dmiss' )*Dback';
    
    end
    
    S00 = S - x_smooth(:,T)*x_smooth(:,T)'-V_smooth(:,:,T) + x_smooth(:,burnIn-1)*x_smooth(:,burnIn-1)'+V_smooth(:,:,burnIn-1);
    S11 = S; 
    
    % M step
    
    BB = S10;
    AA = S00;
    CC = S11;
    
    BB1 = BB(1:d,1:d);
    BB2 = BB(1:d,d+1:2*d);
    BBTilde = [BB1, BB2];
    
    AA11 = AA(1:d, 1:d);
    AA12 = AA(1:d, d+1:2*d);
    AA21 = AA(d+1:2*d, 1:d);
    AA22 = AA(d+1:2*d, d+1:2*d);
    
    Gamma = BB1 - BB2 - AA11 + AA12;
    Theta = AA11 + AA22 - AA12 - AA21;
 
    auxF = Gamma/Theta;
    %auxF(zeroIndex) = 0;
    
%     [~, D] = eig(auxF);
%     if any(abs(diag(D)) >= 1)
%         disp('Found eigenvalue greater than 1!');
%         break;
%     end
    
    FTilde = [Id+auxF, -auxF];
    
    auxQ = (P*CC*P' - BBTilde*FTilde' - FTilde*BBTilde' + FTilde*AA*FTilde')/(T-burnIn+1); 
    auxQ = tril(auxQ)+tril(auxQ,-1)';
    
    R = diag(diag(eps_smooth/(T-burnIn+1)));
    
    % Build system matrices
    
    Q = [auxQ, zeros(d); zeros(d,2*d)];
    F = [FTilde; Id, zeros(d)];
    
%     clc;
%     auxQ
%     auxF
        
%     qVec(:,i+1) = vech2(auxQ);
%     rVec(:,i+1) = diag(R);
%     lVec(:,i+1) = reshape(auxF,d^2,1);
    
    % break for convergence
    deltalog(i)=abs(l(i)-l_old)/abs(l_old);
  
    if deltalog(i)<eps
        str=int2str(i);
        disp(['break for convergence at iteration ', str]);
        fprintf('deltaLog = %s\n', num2str(deltalog(i)));
        c_conv=1;
        break
    end
    l_old=l(i);
    %toc
end

% display if missed convergence
if i==maxiter 
    disp('convergence not reached'); 
    fprintf('deltaLog = %s\n', num2str(deltalog(i)));
end

% reshaping for convergence before maxiter
%if i < maxiter
    
    normR = currfrobR(i);
    normQ = currfrobQ(i);
    normF = currfrobF(i);
%     
%     l=l(isnan(l)==0);
%     
%     qVec=qVec(isnan(qVec)==0);
%     qVec=reshape(qVec,d*(d+1)/2,[]);
%     
%     rVec=rVec(isnan(rVec)==0);
%     rVec=reshape(rVec,d,[]);
%     
%     lVec = lVec(isnan(lVec)==0);
%     lVec=reshape(lVec,d^2,[]);
% 
%end